;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  
;; Spinny thing that chases Mario if he stands on its floor
;;  by leod
;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Palette = $0E
;sprite palette to use, from 08-0F
!SP = $01
;SP1/2 = $00,  SP3/4 = $01
!Tiles = $80
;where the top left tile is located



!FrameDex = $1594     ;current animation frame
                      ;00    = idle
                      ;01-03 = animation

!GoalSpeed = $151C    ;speed the spinner hopes to achieve

!Timer = $1564        ;timer for how long to wait inbetween animation frames (decreases itself)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
print "MAIN ",pc
PHB                     ; \
PHK                     ;  | The same old
PLB                     ; /
JSR MainSprtRt          ;  Jump to sprite code
PLB                     ; Yep
RTL                     ; Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  Return1:
RTS

  MainSprtRt:
JSR SubGFX          ; render graphics

LDA $9D             ; if sprites are locked, return
BNE Return1

JSR SubOffScreen_X3	; handle off screen situation

;; Mario contact
LDA $E4,x       ;x low
SEC
SBC #$02
STA $04
LDA $14E0,x     ;x hi
SBC #$00
STA $0A

LDA $D8,x       ;y low
STA $05
LDA $14D4,x     ;y hi
STA $0B

LDA #$14        ;width  (from 2 pixels to the left to 2 pixels right of edge)
STA $06
LDA #$0F        ;height (from 2 pixels above border to very bottom)
STA $07


JSL $03B664     ;fill in mario clipping into slot B
JSL $03B72B     ;check if the filled in objects intersect
BCC .NoContact

JSR Spiky

  .NoContact
;; Check if Mario is on floor
STZ !GoalSpeed,x

LDA $77
AND #$04
BEQ .NotFloor

LDA #$10
STA $0E     ;prep "mario on yoshi" offset address
LDA $187A
BEQ .NotYoshi

LDA $0E
CLC
ADC #$10
STA $0E

  .NotYoshi
LDA $D8,x
SEC
SBC $0E         ;substract mario size to move pos up basically
STA $0E
LDA $14D4,x
SBC #$00        ;overflow
STA $0F


REP #$20
LDA $0E
CMP $96         ;compare sprite to mario y pos
SEP #$20

BNE .NotFloor   ;if not on same height, skip chase


JSR SubHorzPos
LDA .Goals,y    ;load speed based on which side mario is on
STA !GoalSpeed,x

BRA .NotFloor

.Goals
db $28,$D8

  .NotFloor
;; Close in on goal speed
LDA $B6,x
SEC
SBC !GoalSpeed,x      ;substract goal from current
BEQ .SpeedAchieved    ;if equal, gottem
BMI .INCSpeed         ;if not, adjust

DEC $B6,x
DEC $B6,x
BRA .SpeedAchieved

  .INCSpeed
INC $B6,x
INC $B6,x

  .SpeedAchieved
;; Check for floor and walls
LDA #$02
STA $AA,x

JSL $019138         ;interact with objects
LDA $1588,x
AND #$04            ;if no floor, stop
BEQ .Reverse

LDA $1588,x
AND #$03            ;if ramming walls, stop
BEQ .KeepMoving

  .Reverse
LDA $B6,x
EOR #$FF
INC
STA $B6,x

LDA #$01
STA $1DF9

  .KeepMoving
STZ $AA,x
JSL $018022     ; \ Apply speed to position 
JSL $01801A     ; /

;; Animate
LDA $B6,x
BEQ .NoAnimation

LDA !Timer,x          ;if timer isn't up, don't change anything around
BNE .NoAnimationChange


LDA !FrameDex,x
INC                   ;next frame
CMP #$04
BNE .NoWrap

LDA #$01              ;if frame would be 4 (illegal), wrap around to 01

  .NoWrap
STA !FrameDex,x


LDA $B6,x
BPL .NoInvert

EOR #$FF              ;turn speed positive for math
INC

  .NoInvert
LSR
LSR                   ;/4
STA $0E

LDA #$0C
SEC                   ;0C - speed/4
SBC $0E
STA !Timer,x          ;is how many frames to wait

  .NoAnimationChange
RTS


  .NoAnimation
STZ !FrameDex,x       ;dont animate if no goalspeed
RTS





;;;;
; subroutines
;;;;

Spiky:
;check if mario is above
LDA $140D             ;check if mario is spinjumping
ORA $187A             ;or on yoshi
BNE +
BRA .HurtMario
+

LDA $7D               ;check if hes falling
BPL +
BRA .HurtMario
+

;move up mario's hitbox by his speed to check for top contact
LDA $01
SEC
SBC $7D
STA $01
LDA $09
SBC #$00
STA $09
JSL $03B72B           ;check if the filled in objects intersect
BCC +
BRA .HurtMario        ;if yes, mario isnt on top, so hurt him
+


;move mario up a bit to not instantly get hit again
LDA $96
SEC
SBC #$03
STA $96
LDA $97
SBC #$00
STA $97


LDA #$02              ;spiny sound
STA $1DF9

JSL $01AB99           ;contact graphic

LDA $15
AND #$80              ;launch mario up depending on if hes holding A/B
BEQ .SmallHop

LDA #$B4
STA $7D
RTS

  .SmallHop
LDA #$D0
STA $7D
RTS


  .HurtMario
LDA $187A
BEQ .DoHurt         ;skip this if mario isnt riding yoshi

JSR LoseYoshi

BRA .NoContact

  .DoHurt
JSL $00F5B7         ;hurt mario

  .NoContact
RTS


;;lose yoshi routine (originally by chdata, unstupified by leod)
LoseYoshi:
PHX
LDX $18E2		;Yoshi sprite index +1, would be zero if no Yoshi

LDA #$10
STA $163D,x	;

LDA #$03		;turn off Yoshi drums
STA $1DFA
LDA #$13		;play lose-Yoshi
STA $1DFC

LDA #$02		;set Yoshi to running-away state
STA $C1,x

STZ $187A		;reset riding Yoshi flag
STZ $0DC1		;reset "Mario has Yoshi on OW" flag so he can't keep it anyway
            ;(this is how castle sequences memorize that Yoshi exists)

LDA #$C0		;make Mario jump upwards
STA $7D
STZ $7B			;no x movement

JSR SubHorzPos
LDA .SPEED,y;load yoshi speed indexed by direction
STA $B5,x   ;and make him run

STZ $1593,x		;
STZ $151B,x		;
STZ $18AE		;

LDA #$30		;Mario invincibility timer
STA $1497

PLX			;restore X (sprite index)
RTS

.SPEED
db $E8,$18	;This is Yoshi's X speed table. It is currently Bobo... bo bobo bo!!!


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GFX Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Top = !Tiles+$16

BodyTiles:
db !Tiles+$06,!Tiles+$07,!Tiles+$08

RazorTiles:
db !Tiles+$00,!Tiles+$01,!Tiles+$02
;idle
db !Tiles+$10,!Tiles+$11,!Tiles+$12
db !Tiles+$03,!Tiles+$04,!Tiles+$05
db !Tiles+$13,!Tiles+$14,!Tiles+$15
;WRRRYYYYYYYYYYY

XOffs:
db $FC,$04,$0C



  SubGFX:
JSR GET_DRAW_INFO
  PHX


LDA.b #!Palette-$08<<1|$20|!SP
STA $0303,y  ;top
STA $0307,y
STA $030B,y
STA $030F,y  ;bottom
STA $0313,y
STA $0317,y
STA $031B,y  ;razors
;looks dumb, but literally all tiles have the same props

;;top tile start
LDA $00         ;load sprite x pos
CLC
ADC #$04
STA $0300,y     ;store to oam x pos

LDA $01         ;load sprite y pos
SEC
SBC #$08
STA $0301,y     ;store to oam y pos

LDA.b #!Top
STA $0302,y     ;store to oam tile
;;top tile end

;;bottom tiles start
LDX #$02
  .BottomLoop
PHX

INY : INY : INY : INY

LDA $00
CLC
ADC XOffs,x
STA $0300,y

LDA $01
CLC
ADC #$08
STA $0301,y

LDA BodyTiles,x
STA $0302,y

PLX
DEX
BPL .BottomLoop
;;bottom tiles end

;;razor tiles start
LDX #$02
  .RazorLoop
PHX

INY : INY : INY : INY

LDA $00
CLC
ADC XOffs,x
STA $0300,y

LDA $01
STA $0301,y

STX $0E
LDX $15E9
LDA !FrameDex,x
ASL               ;*2
CLC
ADC !FrameDex,x   ;*3
ADC $0E           ;add current loop
TAX

LDA RazorTiles,x
STA $0302,y

PLX
DEX
BPL .RazorLoop
;;razor tiles end

  PLX

LDY #$00        ; #$00 = 8x8, #$02 = 16x16
LDA #$06        ; tiles drawn; #$00 = 1, #$01 = 2, etc.
JSL $01B7B3     ; handle oam
RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
        ;org $03B75C
 
TABLE1:              db $0C,$1C
TABLE2:              db $01,$02
 
GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
        STZ $15A0,x             ; reset sprite offscreen flag, horizontal
        LDA $E4,x               ; \
        CMP $1A                 ;  | set horizontal offscreen if necessary
        LDA $14E0,x             ;  |
        SBC $1B                 ;  |
        BEQ ON_SCREEN_X         ;  |
        INC $15A0,x             ; /
 
ON_SCREEN_X:         LDA $14E0,x             ; \
        XBA      ;  |
        LDA $E4,x               ;  |
        REP #$20                ;  |
        SEC      ;  |
        SBC $1A                 ;  | mark sprite INVALID if far enough off screen
        CLC      ;  |
        ADC.w #$0040            ;  |
        CMP.w #$0180            ;  |
        SEP #$20                ;  |
        ROL A                   ;  |
        AND #$01                ;  |
        STA $15C4,x             ; /
        BNE INVALID             ;
       
        LDY #$00                ; \ set up LOOP:
        LDA $1662,x             ;  |
        AND #$20                ;  | if not smushed (1662 & 0x20), go through LOOP twice
        BEQ ON_SCREEN_LOOP      ;  | else, go through LOOP once
        INY      ; /
ON_SCREEN_LOOP:      LDA $D8,x               ; \
        CLC      ;  | set vertical offscreen if necessary
        ADC TABLE1,y            ;  |
        PHP      ;  |
        CMP $1C                 ;  | (vert screen boundry)
        ROL $00                 ;  |
        PLP      ;  |
        LDA $14D4,x             ;  |
        ADC #$00                ;  |
        LSR $00                 ;  |
        SBC $1D                 ;  |
        BEQ ON_SCREEN_Y         ;  |
        LDA $186C,x             ;  | (vert offscreen)
        ORA TABLE2,y            ;  |
        STA $186C,x             ;  |
ON_SCREEN_Y:         DEY         ;  |
        BPL ON_SCREEN_LOOP      ; /
 
        LDY $15EA,x             ; get offset to sprite OAM
        LDA $E4,x               ; \
        SEC      ;  |
        SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
        STA $00                 ; /
        LDA $D8,x               ; \
        SEC      ;  |
        SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
        STA $01                 ; /
        RTS      ; RETURN
 
INVALID:             PLA         ; \ RETURN from *main gfx routine* subroutine...
        PLA      ;  |    ...(not just this subroutine)
        RTS      ; /
		    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS RETURN_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubHorzPos
; Stores direction Sprite is in from Mario in Y
; Stores proximity in $0F
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	LDA $95
	SBC $14E0,x
	BPL .HorzIncY
	INY 
.HorzIncY
	RTS 